#ifndef __ResourceManager_H__
#define __ResourceManager_H__


#include <string>
#include <iostream>
#include <fstream>
#include <map>
#include "common.h"


class igzstream;


namespace CHCDemoEngine
{

class SceneEntity;
class Material;
class Geometry;
class Texture;
class Matrix4x4;
class Transform3;


/** Loads a scene and also handles the cleanup
*/
class ResourceManager
{
public:

	~ResourceManager();
	/** Loads a model
	*/
	bool Load(const std::string &filename, SceneEntityContainer &geometry);

protected:
	
	void LoadTextures(igzstream &str);
	void LoadShapes(igzstream &str);
	void LoadSceneEntities(igzstream &str, SceneEntityContainer &entities);

	SceneEntity *LoadSceneEntity(igzstream &str);
	Material *LoadMaterial(igzstream &str);
	Geometry *LoadGeometry(igzstream &str);

	std::map<int, Texture *> mTextureTable;
	std::map<int, Material *> mMaterialTable;
	std::map<int, Geometry *> mGeometryTable;
	//std::map<int, Shape *> mShapesTable;

	// these are kept to be able to delete these resources afterwards
	std::vector<Texture *> mTextures;
	std::vector<Material *> mMaterials;
	std::vector<Geometry *> mGeometry;
	std::vector<Transform3 *> mTrafos;
	
	SceneEntityContainer mSceneEntities;
	ShapeContainer mShapes;
};

}
#endif
