#include "SceneEntity.h"
#include "glInterface.h"
#include "Geometry.h"
#include "Material.h"
#include "RenderState.h"
#include "Shape.h"
#include <Cg/cg.h>
#include <Cg/cgGL.h>


using namespace std;


namespace CHCDemoEngine 
{


SceneEntity::SceneEntity(Transform3 *trafo): mTransform(trafo)
{
}


SceneEntity::~SceneEntity()
{
}


void SceneEntity::GetCurrentLODLevel(ShapeContainer &shapes)
{
	ShapeContainer::const_iterator sit, sit_end;

	if (mLODInfos.empty())
	{
		sit = mShapes.begin(); sit_end = mShapes.end();
	}

	float mDistance = 0.0f;

	int i = 0;

	LODInfoContainer::const_iterator lid, lid_end = mLODInfos.end();

	for (lid = mLODInfos.begin(); lid != lid_end; ++ lid)
	{
		LODInfo *lodInfo = *lid;

		if (lodInfo->mSquaredDistance > mDistance)
		{
			sit = lodInfo->mShapes.begin(); 
			sit_end = lodInfo->mShapes.end();
			break;
		}
	}

	for (; sit != sit_end; ++ sit)
	{
		shapes.push_back(*sit);
	}
}


void SceneEntity::Render(RenderState *state)
{
	//if (mMaterial) mMaterial->Render(state);
	float mDistance = 0.0f;

	static ShapeContainer shapes;
	shapes.clear();

	GetCurrentLODLevel(shapes);

	ShapeContainer::const_iterator sit, sit_end = shapes.end();

	for (sit = shapes.begin(); sit != sit_end; ++ sit)
	{
		(*sit)->Render(state);
	}
}


void SceneEntity::AddShape(Shape *shape)
{
	mShapes.push_back(shape);
}


void SceneEntity::SetTransform(Transform3 *trafo)
{
	mTransform = trafo;
}


void SceneEntity::SetLastRendered(int lastRendered)
{
	mLastRendered = lastRendered;
}


int SceneEntity::GetLastRendered() const
{
	return mLastRendered;
}


int SceneEntity::CountNumTriangles()
{
	int numTriangles = 0;
	static ShapeContainer shapes;
	shapes.clear();

	GetCurrentLODLevel(shapes);

	ShapeContainer::const_iterator sit, sit_end = shapes.end();

	for (sit = shapes.begin(); sit != sit_end; ++ sit)
	{
		numTriangles += (*sit)->GetGeometry()->GetNumTriangles();
	}

	return numTriangles;
}


}